@@ -1,8 +1,55 @@
+def get(obj, name):
+    for item in name.split('.'):
+        obj = getattr(obj,item)
+    return obj
+
+def getkey(obj, name):
+    for item in name.split('.'):
+        obj = obj[item]
+    return obj
+
+def getindex(obj, name):
+    for item in name.split('.'):
+        item = int(item)
+        if not len(obj) >= item:
+            return False, 0
+        obj = obj[item]
+    return True, obj
+
 class Assertion(object):
     def __init__(self, obj):
         self.subject = obj
         self.Not = False
         self.length = False
+        self.deep = False
+
+    @property
+    def obj(self):
+        if self.length:
+            return len(self.subject)
+        return self.subject
+
+    def Return(self, value, *args, **kwargs):
+        subject = self.obj
+        res = subject(*args, **kwargs) == value
+        if not self.toggle(res):
+            message = "Object [{0}] doesn't return [{1}]"
+            if self.Not:
+                message = "Object [{0}] returned [{1}]"
+            raise AssertionError(message.format(subject, value))
+
+    def element(self, name):
+        message = "Object [{0}] doesn't have element at [{1}]"
+        subject = self.obj
+        if self.deep:
+            target, res = getindex(subject, name)
+            if not self.toggle(target):
+                raise AssertionError(message.format(subject, name))
+            return res
+
+        if not len(subject) >= name:
+            raise AssertionError(message.format(subject, name))
+        return subject[name]
 
     def ok(self):
         subject = self.obj
@@ -92,9 +139,10 @@ class Assertion(object):
                     )
 
     def method(self, name):
-        subject = self.obj
-        self.have(name)
-        if not self.toggle(callable(getattr(subject,name))):
+        res = self.have(name)
+        if not self.deep:
+            res = getattr(self.obj, name)
+        if not self.toggle(callable(res)):
             message = "Object [{0}] does not have method {1}."
             if self.Not:
                 message = "Object [{0}] has method {1}."
@@ -102,13 +150,6 @@ class Assertion(object):
                     message.format(repr(subject), repr(name))
                     )
 
-    @property
-    def obj(self):
-        if self.length:
-            #self.length = False
-            return len(self.subject)
-        return self.subject
-
     def lt(self, value):
         subject = self.obj
         if not self.toggle(subject < value):
@@ -162,11 +203,15 @@ class Assertion(object):
 
     def have(self, attribute):
         subject = self.obj
-        if not self.toggle(hasattr(subject, attribute)):
+        res = hasattr(subject, attribute)
+        if self.deep:
+            res = get(subject, attribute)
+        if not self.toggle(res):
             raise AssertionError(
                     "Object [{0}] doesn't have attribute [{1}]"\
                             .format(repr(subject), attribute)
                     )
+        return res
 
     def equals(self, value):
         subject = self.obj
